
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import java.io.FileNotFoundException;
import java.io.IOException;


public class PersistentMemory {
    // Implements the persistent memory of a smartcard.  This is the only
    //    memory that survives from one activation of the card to the next.
    // The persistent memory is an array of <Size> bytes, which can be
    //    accessed using the get and set methods.
    
    public static final int Size = 65536;

    private String fileName;
    private byte[] state = new byte[Size];

    public PersistentMemory(String stateFilename) 
                                   throws FileNotFoundException, IOException {
	fileName = stateFilename;
	InputStream is = new FileInputStream(fileName);
	is.read(state);
	is.close();
    }

    public void save() throws FileNotFoundException, IOException {
	OutputStream os = new FileOutputStream(fileName);
	os.write(state);
	os.close();
    }

    public byte get(int offset) {
	if((offset<0) || (offset>=Size)) throw new IndexOutOfBoundsException();
	return state[offset];
    }
	
    public void get(int offset, int len, byte[] arr, int arrOffset) {
	for(int i=0; i<len; ++i) {
	    arr[arrOffset+i] = get(offset+i);
	}
    }

    private void put_nosave(int offset, byte value) {
	if((offset<0) || (offset>=Size)) throw new IndexOutOfBoundsException();
	state[offset] = value;
    }

    public void put(int offset, byte value) throws IOException {
	put_nosave(offset, value);
	save();
    }

    public void put(int offset, int len, byte[] arr, int arrOffset) 
                                                         throws IOException {
	for(int i=0; i<len; ++i) {
	    put_nosave(offset+i, arr[arrOffset+i]);
	}
	save();
    }
}